"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const observable_1 = require("@reactive-js/observable");
const pipe_1 = require("@reactive-js/pipe");
class AsyncEnumeratorImpl extends observable_1.AbstractDelegatingSubscriber {
    constructor(delegate, observable) {
        super(delegate);
        this.observable = observable;
        this.isSynchronous = false;
        this.add(observable);
    }
    get subscriberCount() {
        return this.observable.subscriberCount;
    }
    notify(req) {
        this.delegate.notify(req);
    }
    dispatch(req) {
        this.delegate.dispatch(req);
    }
    subscribe(subscriber) {
        this.observable.subscribe(subscriber);
    }
}
exports.AsyncEnumeratorImpl = AsyncEnumeratorImpl;
exports.createAsyncEnumerator = (op, scheduler, replayCount) => {
    const subject = observable_1.createSubject(scheduler);
    const safeSubscriber = observable_1.toSafeSubscriber(subject);
    const observable = pipe_1.pipe(subject, op, observable_1.publish(scheduler, replayCount));
    return new AsyncEnumeratorImpl(safeSubscriber, observable);
};
class AsyncEnumerableImpl {
    constructor(op) {
        this.op = op;
    }
    enumerateAsync(scheduler, replayCount) {
        return exports.createAsyncEnumerator(this.op, scheduler, replayCount);
    }
}
exports.AsyncEnumerableImpl = AsyncEnumerableImpl;
exports.createAsyncEnumerable = (op) => new AsyncEnumerableImpl(op);
//# sourceMappingURL=createAsyncEnumerable.js.map