import { AbstractDelegatingSubscriber, createSubject, publish, toSafeSubscriber, } from "@reactive-js/observable";
import { pipe } from "@reactive-js/pipe";
export class AsyncEnumeratorImpl extends AbstractDelegatingSubscriber {
    constructor(delegate, observable) {
        super(delegate);
        this.observable = observable;
        this.isSynchronous = false;
        this.add(observable);
    }
    get subscriberCount() {
        return this.observable.subscriberCount;
    }
    notify(req) {
        this.delegate.notify(req);
    }
    dispatch(req) {
        this.delegate.dispatch(req);
    }
    subscribe(subscriber) {
        this.observable.subscribe(subscriber);
    }
}
export const createAsyncEnumerator = (op, scheduler, replayCount) => {
    const subject = createSubject(scheduler);
    const safeSubscriber = toSafeSubscriber(subject);
    const observable = pipe(subject, op, publish(scheduler, replayCount));
    return new AsyncEnumeratorImpl(safeSubscriber, observable);
};
export class AsyncEnumerableImpl {
    constructor(op) {
        this.op = op;
    }
    enumerateAsync(scheduler, replayCount) {
        return createAsyncEnumerator(this.op, scheduler, replayCount);
    }
}
export const createAsyncEnumerable = (op) => new AsyncEnumerableImpl(op);
//# sourceMappingURL=createAsyncEnumerable.js.map