import { map, onNotify, ignoreElements, merge, using, } from "@reactive-js/observable";
import { AsyncEnumerableImpl } from "./createAsyncEnumerable";
import { pipe } from "@reactive-js/pipe";
class LiftedAsyncEnumerable extends AsyncEnumerableImpl {
    constructor(op, src, obsOps, reqOps) {
        super(op);
        this.src = src;
        this.obsOps = obsOps;
        this.reqOps = reqOps;
    }
}
const reducer = (acc, next) => next(acc);
const createFactory = (obsOps, reqOps, requests) => (enumerator) => {
    const observable = obsOps.reduce(reducer, enumerator);
    const mapRequest = (req) => reqOps.reduce(reducer, req);
    const onRequest = pipe(requests, map(mapRequest), onNotify((req) => enumerator.dispatch(req)), ignoreElements());
    return merge(observable, onRequest);
};
const liftImpl = (enumerable, obsOps, reqOps) => {
    const src = enumerable instanceof LiftedAsyncEnumerable ? enumerable.src : enumerable;
    const op = (requests) => using(scheduler => src.enumerateAsync(scheduler), createFactory(obsOps, reqOps, requests));
    return new LiftedAsyncEnumerable(op, src, obsOps, reqOps);
};
export const lift = (op) => enumerable => {
    const obsOps = enumerable instanceof LiftedAsyncEnumerable
        ? [...enumerable.obsOps, op]
        : [op];
    const reqOps = enumerable instanceof LiftedAsyncEnumerable ? enumerable.reqOps : [];
    return liftImpl(enumerable, obsOps, reqOps);
};
export const liftReq = (op) => enumerable => {
    const obsOps = enumerable instanceof LiftedAsyncEnumerable ? enumerable.obsOps : [];
    const reqOps = enumerable instanceof LiftedAsyncEnumerable
        ? [op, ...enumerable.reqOps]
        : [op];
    return liftImpl(enumerable, obsOps, reqOps);
};
//# sourceMappingURL=lift.js.map